@@ -1,77 +1,75 @@
+from __future__ import annotations
+
 import os
-from posixpath import normpath
 import re
 import warnings
+from posixpath import normpath
+from typing import Callable
 from wsgiref.headers import Headers
 from wsgiref.util import FileWrapper
 
 from .media_types import MediaTypes
-from .responders import StaticFile, MissingFileError, IsDirectoryError, Redirect
-from .string_utils import (
-    decode_if_byte_string,
-    decode_path_info,
-    ensure_leading_trailing_slash,
-)
+from .responders import IsDirectoryError
+from .responders import MissingFileError
+from .responders import Redirect
+from .responders import StaticFile
+from .string_utils import decode_if_byte_string
+from .string_utils import decode_path_info
+from .string_utils import ensure_leading_trailing_slash
 
 
-class WhiteNoise(object):
+class WhiteNoise:
 
     # Ten years is what nginx sets a max age if you use 'expires max;'
     # so we'll follow its lead
     FOREVER = 10 * 365 * 24 * 60 * 60
 
-    # Attributes that can be set by keyword args in the constructor
-    config_attrs = (
-        "autorefresh",
-        "max_age",
-        "allow_all_origins",
-        "charset",
-        "mimetypes",
-        "add_headers_function",
-        "index_file",
-        "immutable_file_test",
-    )
-    # Re-check the filesystem on every request so that any changes are
-    # automatically picked up. NOTE: For use in development only, not supported
-    # in production
-    autorefresh = False
-    max_age = 60
-    # Set 'Access-Control-Allow-Orign: *' header on all files.
-    # As these are all public static files this is safe (See
-    # https://www.w3.org/TR/cors/#security) and ensures that things (e.g
-    # webfonts in Firefox) still work as expected when your static files are
-    # served from a CDN, rather than your primary domain.
-    allow_all_origins = True
-    charset = "utf-8"
-    # Custom mime types
-    mimetypes = None
-    # Callback for adding custom logic when setting headers
-    add_headers_function = None
-    # Name of index file (None to disable index support)
-    index_file = None
+    def __init__(
+        self,
+        application,
+        root=None,
+        prefix=None,
+        *,
+        # Re-check the filesystem on every request so that any changes are
+        # automatically picked up. NOTE: For use in development only, not supported
+        # in production
+        autorefresh: bool = False,
+        max_age: int | None = 60,  # seconds
+        # Set 'Access-Control-Allow-Origin: *' header on all files.
+        # As these are all public static files this is safe (See
+        # https://www.w3.org/TR/cors/#security) and ensures that things (e.g
+        # webfonts in Firefox) still work as expected when your static files are
+        # served from a CDN, rather than your primary domain.
+        allow_all_origins: bool = True,
+        charset: str = "utf-8",
+        mimetypes: dict[str, str] | None = None,
+        add_headers_function: Callable[[Headers, str, str], None] | None = None,
+        index_file: str | bool | None = None,
+        immutable_file_test: Callable | str | None = None,
+    ):
+        self.autorefresh = autorefresh
+        self.max_age = max_age
+        self.allow_all_origins = allow_all_origins
+        self.charset = charset
+        self.add_headers_function = add_headers_function
+        if index_file is True:
+            self.index_file: str | None = "index.html"
+        elif isinstance(index_file, str):
+            self.index_file = index_file
+        else:
+            self.index_file = None
 
-    def __init__(self, application, root=None, prefix=None, **kwargs):
-        for attr in self.config_attrs:
-            try:
-                value = kwargs.pop(attr)
-            except KeyError:
-                pass
+        if immutable_file_test is not None:
+            if not callable(immutable_file_test):
+                regex = re.compile(immutable_file_test)
+                self.immutable_file_test = lambda path, url: bool(regex.search(url))
             else:
-                value = decode_if_byte_string(value)
-                setattr(self, attr, value)
-        if kwargs:
-            raise TypeError(
-                "Unexpected keyword argument '{0}'".format(list(kwargs.keys())[0])
-            )
-        self.media_types = MediaTypes(extra_types=self.mimetypes)
+                self.immutable_file_test = immutable_file_test
+
+        self.media_types = MediaTypes(extra_types=mimetypes)
         self.application = application
         self.files = {}
         self.directories = []
-        if self.index_file is True:
-            self.index_file = "index.html"
-        if not callable(self.immutable_file_test):
-            regex = re.compile(self.immutable_file_test)
-            self.immutable_file_test = lambda path, url: bool(regex.search(url))
         if root is not None:
             self.add_files(root, prefix)
 
@@ -89,7 +87,7 @@ class WhiteNoise(object):
     @staticmethod
     def serve(static_file, environ, start_response):
         response = static_file.get_response(environ["REQUEST_METHOD"], environ)
-        status_line = "{} {}".format(response.status, response.status.phrase)
+        status_line = f"{response.status} {response.status.phrase}"
         start_response(status_line, list(response.headers))
         if response.file is not None:
             file_wrapper = environ.get("wsgi.file_wrapper", FileWrapper)
@@ -112,7 +110,7 @@ class WhiteNoise(object):
             if os.path.isdir(root):
                 self.update_files_dictionary(root, prefix)
             else:
-                warnings.warn(u"No directory at: {}".format(root))
+                warnings.warn(f"No directory at: {root}")
 
     def update_files_dictionary(self, root, prefix):
         # Build a mapping from paths to the results of `os.stat` calls
@@ -127,7 +125,7 @@ class WhiteNoise(object):
     def add_file_to_dictionary(self, url, path, stat_cache=None):
         if self.is_compressed_variant(path, stat_cache=stat_cache):
             return
-        if self.index_file and url.endswith("/" + self.index_file):
+        if self.index_file is not None and url.endswith("/" + self.index_file):
             index_url = url[: -len(self.index_file)]
             index_no_slash = index_url.rstrip("/")
             self.files[url] = self.redirect(url, index_url)
@@ -138,7 +136,7 @@ class WhiteNoise(object):
 
     def find_file(self, url):
         # Optimization: bail early if the URL can never match a file
-        if not self.index_file and url.endswith("/"):
+        if self.index_file is None and url.endswith("/"):
             return
         if not self.url_is_canonical(url):
             return
@@ -158,25 +156,23 @@ class WhiteNoise(object):
     def find_file_at_path(self, path, url):
         if self.is_compressed_variant(path):
             raise MissingFileError(path)
-        if self.index_file:
-            return self.find_file_at_path_with_indexes(path, url)
-        else:
-            return self.get_static_file(path, url)
 
-    def find_file_at_path_with_indexes(self, path, url):
-        if url.endswith("/"):
-            path = os.path.join(path, self.index_file)
-            return self.get_static_file(path, url)
-        elif url.endswith("/" + self.index_file):
-            if os.path.isfile(path):
-                return self.redirect(url, url[: -len(self.index_file)])
-        else:
-            try:
+        if self.index_file is not None:
+            if url.endswith("/"):
+                path = os.path.join(path, self.index_file)
                 return self.get_static_file(path, url)
-            except IsDirectoryError:
-                if os.path.isfile(os.path.join(path, self.index_file)):
-                    return self.redirect(url, url + "/")
-        raise MissingFileError(path)
+            elif url.endswith("/" + self.index_file):
+                if os.path.isfile(path):
+                    return self.redirect(url, url[: -len(self.index_file)])
+            else:
+                try:
+                    return self.get_static_file(path, url)
+                except IsDirectoryError:
+                    if os.path.isfile(os.path.join(path, self.index_file)):
+                        return self.redirect(url, url + "/")
+            raise MissingFileError(path)
+
+        return self.get_static_file(path, url)
 
     @staticmethod
     def url_is_canonical(url):
@@ -210,7 +206,7 @@ class WhiteNoise(object):
         self.add_cache_headers(headers, path, url)
         if self.allow_all_origins:
             headers["Access-Control-Allow-Origin"] = "*"
-        if self.add_headers_function:
+        if self.add_headers_function is not None:
             self.add_headers_function(headers, path, url)
         return StaticFile(
             path,
@@ -229,11 +225,11 @@ class WhiteNoise(object):
 
     def add_cache_headers(self, headers, path, url):
         if self.immutable_file_test(path, url):
-            headers["Cache-Control"] = "max-age={0}, public, immutable".format(
+            headers["Cache-Control"] = "max-age={}, public, immutable".format(
                 self.FOREVER
             )
         elif self.max_age is not None:
-            headers["Cache-Control"] = "max-age={0}, public".format(self.max_age)
+            headers["Cache-Control"] = f"max-age={self.max_age}, public"
 
     def immutable_file_test(self, path, url):
         """
@@ -254,9 +250,9 @@ class WhiteNoise(object):
         elif from_url == to_url + self.index_file:
             relative_url = "./"
         else:
-            raise ValueError("Cannot handle redirect: {} > {}".format(from_url, to_url))
+            raise ValueError(f"Cannot handle redirect: {from_url} > {to_url}")
         if self.max_age is not None:
-            headers = {"Cache-Control": "max-age={0}, public".format(self.max_age)}
+            headers = {"Cache-Control": f"max-age={self.max_age}, public"}
         else:
             headers = {}
         return Redirect(relative_url, headers=headers)
